@@ -16,6 +16,7 @@ config = {
     "dispatcher_image_pull_policy": None,
     "save_keys_in_global_outputs": True,
     "mode": "default",
+    "lineage": None,
 }
 
 
@@ -62,6 +63,7 @@ s3_config = {
     "repo_prefix": "",
     "prefix": "",
     "storage_client": None,
+    "extra_prefixes": [],
 }
 
 
@@ -84,5 +86,6 @@ def set_s3_config(
         repo_prefix: prefix of artifact repository, parsed from repo_key
         prefix: prefix of storage key
         storage_client: client for plugin storage backend
+        extra_prefixes: extra prefixes ignored by auto-prefixing
     """
     s3_config.update(kwargs)
