@@ -2,7 +2,7 @@ import os
 import tempfile
 from collections import UserDict
 from copy import deepcopy
-from typing import Any, Dict, List, Union
+from typing import Optional, Any, Dict, List, Union
 
 import jsonpickle
 
@@ -61,6 +61,14 @@ class InputArtifacts(AutonamedDict):
             else:
                 self.template.inputs.parameters["dflow_%s_path_list" % key] = \
                     InputParameter(value=[])
+        if config["lineage"] and self.template is not None and \
+                key[:6] != "dflow_":
+            if isinstance(value.source, S3Artifact):
+                self.template.inputs.parameters["dflow_%s_urn" % key] = \
+                    InputParameter(value=value.source.urn)
+            else:
+                self.template.inputs.parameters["dflow_%s_urn" % key] = \
+                    InputParameter(value="")
 
     def set_template(self, template):
         super().set_template(template)
@@ -74,6 +82,16 @@ class InputArtifacts(AutonamedDict):
                     self.template.inputs.parameters["dflow_%s_path_list"
                                                     % name] = InputParameter(
                         value=[])
+        if config["lineage"]:
+            for name, art in self.items():
+                if name[:6] == "dflow_":
+                    continue
+                if isinstance(art.source, S3Artifact):
+                    self.template.inputs.parameters["dflow_%s_urn" % name] = \
+                        InputParameter(value=art.source.urn)
+                else:
+                    self.template.inputs.parameters["dflow_%s_urn" % name] = \
+                        InputParameter(value="")
 
 
 class OutputParameters(AutonamedDict):
@@ -90,6 +108,10 @@ class OutputArtifacts(AutonamedDict):
             self.template.outputs.parameters["dflow_%s_path_list" % key] = \
                 OutputParameter(value=[])
             value.handle_path_list()
+        if config["lineage"] and self.template is not None:
+            self.template.outputs.parameters["dflow_%s_urn" % key] = \
+                OutputParameter(value="")
+            value.handle_urn()
 
     def set_template(self, template):
         super().set_template(template)
@@ -98,6 +120,11 @@ class OutputArtifacts(AutonamedDict):
                 self.template.outputs.parameters["dflow_%s_path_list" % name]\
                     = OutputParameter(value=[])
                 art.handle_path_list()
+        if config["lineage"]:
+            for name, art in self.items():
+                self.template.outputs.parameters["dflow_%s_urn" % name]\
+                    = OutputParameter(value="")
+                art.handle_urn()
 
 
 class ArgoVar:
@@ -222,8 +249,8 @@ class PVC:
             name: str,
             subpath: str,
             size: str = "1Gi",
-            storage_class: str = None,
-            access_modes: List[str] = None,
+            storage_class: Optional[str] = None,
+            access_modes: Optional[List[str]] = None,
     ) -> None:
         self.name = name
         self.subpath = subpath
@@ -246,12 +273,12 @@ class InputParameter(ArgoVar):
 
     def __init__(
             self,
-            name: str = None,
+            name: Optional[str] = None,
             step=None,
             template=None,
-            type: Any = None,
+            type: Optional[Any] = None,
             save_as_artifact: bool = False,
-            path: str = None,
+            path: Optional[str] = None,
             source: Union["InputArtifact",
                           "OutputArtifact", S3Artifact] = None,
             **kwargs,
@@ -391,16 +418,16 @@ class InputArtifact(ArgoVar):
 
     def __init__(
             self,
-            path: str = None,
-            name: str = None,
+            path: Optional[str] = None,
+            name: Optional[str] = None,
             step=None,
             template=None,
             optional: bool = False,
-            type: Any = None,
+            type: Optional[Any] = None,
             source: Union[str, "InputArtifact",
                           "OutputArtifact", S3Artifact] = None,
-            mode: int = None,
-            sub_path: str = None,
+            mode: Optional[int] = None,
+            sub_path: Optional[str] = None,
             **kwargs,
     ) -> None:
         self.path = path
@@ -451,6 +478,9 @@ class InputArtifact(ArgoVar):
         return self.template.inputs.parameters["dflow_%s_path_list" %
                                                self.name]
 
+    def get_urn_parameter(self):
+        return self.template.inputs.parameters["dflow_%s_urn" % self.name]
+
     def convert_to_argo(self):
         if self.path in NotAllowedInputArtifactPath:
             raise RuntimeError(
@@ -503,14 +533,14 @@ class OutputParameter(ArgoVar):
 
     def __init__(
             self,
-            value_from_path: str = None,
+            value_from_path: Optional[str] = None,
             value_from_parameter: Union[InputParameter,
                                         "OutputParameter"] = None,
-            name: str = None,
+            name: Optional[str] = None,
             step=None,
             template=None,
-            type: Any = None,
-            global_name: str = None,
+            type: Optional[Any] = None,
+            global_name: Optional[str] = None,
             value_from_expression: Union[str, IfExpression] = None,
             save_as_artifact: bool = False,
             **kwargs,
@@ -697,15 +727,15 @@ class OutputArtifact(ArgoVar):
 
     def __init__(
             self,
-            path: os.PathLike = None,
+            path: Optional[os.PathLike] = None,
             _from: Union[InputArtifact, "OutputArtifact"] = None,
-            name: str = None,
+            name: Optional[str] = None,
             step=None,
             template=None,
-            type: Any = None,
+            type: Optional[Any] = None,
             save: List[Union[PVC, S3Artifact]] = None,
             archive: str = "default",
-            global_name: str = None,
+            global_name: Optional[str] = None,
             from_expression: Union[IfExpression, str] = None,
             **kwargs,
     ) -> None:
@@ -757,6 +787,8 @@ class OutputArtifact(ArgoVar):
         if config["save_path_as_parameter"] and key in ["_from",
                                                         "from_expression"]:
             self.handle_path_list()
+        if config["lineage"] and key in ["_from", "from_expression"]:
+            self.handle_urn()
 
     def handle_path_list(self):
         if self.template is not None:
@@ -778,9 +810,32 @@ class OutputArtifact(ArgoVar):
                     _else=self.from_expression._else.get_path_list_parameter(),
                 )
 
+    def handle_urn(self):
+        if self.template is not None:
+            if hasattr(self, "_from") and self._from is not None:
+                self.template.outputs.parameters["dflow_%s_urn" %
+                                                 self.name].default = ""
+                self.template.outputs.parameters[
+                    "dflow_%s_urn" % self.name].value_from_parameter = \
+                    self._from.get_urn_parameter()
+            elif hasattr(self, "from_expression") and self.from_expression is \
+                    not None:
+                self.template.outputs.parameters["dflow_%s_urn" %
+                                                 self.name].default = ""
+                self.template.outputs.parameters[
+                    "dflow_%s_urn" % self.name].value_from_expression = \
+                    if_expression(
+                    _if=self.from_expression._if,
+                    _then=self.from_expression._then.get_urn_parameter(),
+                    _else=self.from_expression._else.get_urn_parameter(),
+                )
+
     def get_path_list_parameter(self):
         return self.step.outputs.parameters["dflow_%s_path_list" % self.name]
 
+    def get_urn_parameter(self):
+        return self.step.outputs.parameters["dflow_%s_urn" % self.name]
+
     def __repr__(self):
         from .task import Task
         if self.redirect is not None:
