@@ -1,5 +1,5 @@
 from copy import deepcopy
-from typing import Dict, List, Union
+from typing import Optional, Dict, List, Union
 
 from .config import config as global_config
 from .config import s3_config
@@ -29,11 +29,11 @@ except Exception:
 class OPTemplate:
     def __init__(
             self,
-            name: str = None,
-            inputs: Inputs = None,
-            outputs: Outputs = None,
-            memoize_key: str = None,
-            pvcs: List[PVC] = None,
+            name: Optional[str] = None,
+            inputs: Optional[Inputs] = None,
+            outputs: Optional[Outputs] = None,
+            memoize_key: Optional[str] = None,
+            pvcs: Optional[List[PVC]] = None,
             annotations: Dict[str, str] = None,
             **kwargs,
     ) -> None:
@@ -130,26 +130,26 @@ class ScriptOPTemplate(OPTemplate):
 
     def __init__(
             self,
-            name: str = None,
-            inputs: Inputs = None,
-            outputs: Outputs = None,
-            memoize_key: str = None,
-            pvcs: List[PVC] = None,
+            name: Optional[str] = None,
+            inputs: Optional[Inputs] = None,
+            outputs: Optional[Outputs] = None,
+            memoize_key: Optional[str] = None,
+            pvcs: Optional[List[PVC]] = None,
             annotations: Dict[str, str] = None,
-            image: str = None,
+            image: Optional[str] = None,
             command: Union[str, List[str]] = None,
-            script: str = None,
-            volumes: List[V1Volume] = None,
-            mounts: List[V1VolumeMount] = None,
+            script: Optional[str] = None,
+            volumes: Optional[List[V1Volume]] = None,
+            mounts: Optional[List[V1VolumeMount]] = None,
             init_progress: str = "0/1",
-            timeout: str = None,
-            retry_strategy: V1alpha1RetryStrategy = None,
-            resource: V1alpha1ResourceTemplate = None,
-            image_pull_policy: str = None,
+            timeout: Optional[str] = None,
+            retry_strategy: Optional[V1alpha1RetryStrategy] = None,
+            resource: Optional[V1alpha1ResourceTemplate] = None,
+            image_pull_policy: Optional[str] = None,
             requests: Dict[str, str] = None,
             limits: Dict[str, str] = None,
             envs: Dict[str, str] = None,
-            init_containers: List[V1alpha1UserContainer] = None,
+            init_containers: Optional[List[V1alpha1UserContainer]] = None,
             **kwargs,
     ) -> None:
         super().__init__(name=name, inputs=inputs, outputs=outputs,
@@ -257,25 +257,25 @@ class ShellOPTemplate(ScriptOPTemplate):
 
     def __init__(
         self,
-        name: str = None,
-        inputs: Inputs = None,
-        outputs: Outputs = None,
-        memoize_key: str = None,
-        pvcs: List[PVC] = None,
+        name: Optional[str] = None,
+        inputs: Optional[Inputs] = None,
+        outputs: Optional[Outputs] = None,
+        memoize_key: Optional[str] = None,
+        pvcs: Optional[List[PVC]] = None,
         annotations: Dict[str, str] = None,
-        image: str = None,
+        image: Optional[str] = None,
         command: Union[str, List[str]] = None,
-        script: str = None,
-        volumes: List[V1Volume] = None,
-        mounts: List[V1VolumeMount] = None,
+        script: Optional[str] = None,
+        volumes: Optional[List[V1Volume]] = None,
+        mounts: Optional[List[V1VolumeMount]] = None,
         init_progress: str = "0/1",
-        timeout: str = None,
-        retry_strategy: V1alpha1RetryStrategy = None,
-        image_pull_policy: str = None,
+        timeout: Optional[str] = None,
+        retry_strategy: Optional[V1alpha1RetryStrategy] = None,
+        image_pull_policy: Optional[str] = None,
         requests: Dict[str, str] = None,
         limits: Dict[str, str] = None,
         envs: Dict[str, str] = None,
-        init_containers: List[V1alpha1UserContainer] = None,
+        init_containers: Optional[List[V1alpha1UserContainer]] = None,
         **kwargs,
     ) -> None:
         if command is None:
@@ -319,25 +319,25 @@ class PythonScriptOPTemplate(ScriptOPTemplate):
 
     def __init__(
         self,
-        name: str = None,
-        inputs: Inputs = None,
-        outputs: Outputs = None,
-        memoize_key: str = None,
-        pvcs: List[PVC] = None,
+        name: Optional[str] = None,
+        inputs: Optional[Inputs] = None,
+        outputs: Optional[Outputs] = None,
+        memoize_key: Optional[str] = None,
+        pvcs: Optional[List[PVC]] = None,
         annotations: Dict[str, str] = None,
-        image: str = None,
+        image: Optional[str] = None,
         command: Union[str, List[str]] = None,
-        script: str = None,
-        volumes: List[V1Volume] = None,
-        mounts: List[V1VolumeMount] = None,
+        script: Optional[str] = None,
+        volumes: Optional[List[V1Volume]] = None,
+        mounts: Optional[List[V1VolumeMount]] = None,
         init_progress: str = "0/1",
-        timeout: str = None,
-        retry_strategy: V1alpha1RetryStrategy = None,
-        image_pull_policy: str = None,
+        timeout: Optional[str] = None,
+        retry_strategy: Optional[V1alpha1RetryStrategy] = None,
+        image_pull_policy: Optional[str] = None,
         requests: Dict[str, str] = None,
         limits: Dict[str, str] = None,
         envs: Dict[str, str] = None,
-        init_containers: List[V1alpha1UserContainer] = None,
+        init_containers: Optional[List[V1alpha1UserContainer]] = None,
         **kwargs,
     ) -> None:
         if command is None:
