@@ -2,9 +2,10 @@ import json
 import os
 from copy import deepcopy
 from getpass import getpass
+from typing import Optional
 
 from ..context import Context
-from ..executor import Executor
+from ..executor import Executor, render_script_with_tmp_root
 from ..op_template import PythonScriptOPTemplate, ShellOPTemplate
 from ..utils import randstr, s3_config
 from ..workflow import Workflow
@@ -58,8 +59,8 @@ class BohriumExecutor(Executor):
 
     def __init__(
             self,
-            executor: str = None,
-            extra: dict = None,
+            executor: Optional[str] = None,
+            extra: Optional[dict] = None,
     ) -> None:
         self.executor = executor
         self.extra = extra
@@ -78,8 +79,8 @@ class BohriumExecutor(Executor):
                 self.extra) if isinstance(self.extra, dict) else self.extra
         if self.executor == "bohrium_v2" and template.annotations[
                 "workflow.dp.tech/executor"] != "bohrium_v2":
-            new_template.script = new_template.script.replace(
-                "/tmp", "$(pwd)/tmp")
+            new_template.script = render_script_with_tmp_root(template,
+                                                              "$(pwd)/tmp")
             if isinstance(template, ShellOPTemplate):
                 new_template.script = "mkdir -p tmp\n" + new_template.script
             if isinstance(template, PythonScriptOPTemplate):
@@ -103,12 +104,12 @@ class BohriumContext(Context):
 
     def __init__(
             self,
-            username: str = None,
-            password: str = None,
-            bohrium_url: str = None,
-            executor: str = None,
-            extra: dict = None,
-            authorization: str = None,
+            username: Optional[str] = None,
+            password: Optional[str] = None,
+            bohrium_url: Optional[str] = None,
+            executor: Optional[str] = None,
+            extra: Optional[dict] = None,
+            authorization: Optional[str] = None,
     ) -> None:
         self.bohrium_url = bohrium_url if bohrium_url is not None else \
             config["bohrium_url"]
@@ -146,8 +147,8 @@ class BohriumContext(Context):
             new_template.annotations["workflow.dp.tech/executor"] = \
                 self.executor
             if self.executor == "bohrium_v2":
-                new_template.script = new_template.script.replace(
-                    "/tmp", "$(pwd)/tmp")
+                new_template.script = render_script_with_tmp_root(template,
+                                                                  "$(pwd)/tmp")
                 if isinstance(template, ShellOPTemplate):
                     new_template.script = "mkdir -p tmp\n" + \
                         new_template.script
@@ -162,14 +163,16 @@ class BohriumContext(Context):
 class TiefblueClient:
     def __init__(
             self,
-            bohrium_url: str = None,
-            username: str = None,
-            password: str = None,
-            authorization: str = None,
-            project_id: str = None,
-            token: str = None,
-            prefix: str = None,
-            tiefblue_url: str = None,
+            bohrium_url: Optional[str] = None,
+            username: Optional[str] = None,
+            password: Optional[str] = None,
+            authorization: Optional[str] = None,
+            project_id: Optional[str] = None,
+            token: Optional[str] = None,
+            prefix: Optional[str] = None,
+            sharePath: Optional[str] = None,
+            userSharePath: Optional[str] = None,
+            tiefblue_url: Optional[str] = None,
     ) -> None:
         # only set s3_config["storage_client"] once
         if isinstance(s3_config["storage_client"], self.__class__):
@@ -190,10 +193,14 @@ class TiefblueClient:
             config["tiefblue_url"]
         self.token = token
         self.prefix = prefix
+        self.sharePath = sharePath
+        self.userSharePath = userSharePath
         if self.token is None:
             self.get_token()
         s3_config["repo_type"] = "oss"
         s3_config["prefix"] = self.prefix
+        s3_config["extra_prefixes"].append(self.sharePath)
+        s3_config["extra_prefixes"].append(self.userSharePath)
 
     def to_dict(self):
         retained_keys = ["bohrium_url",
@@ -202,7 +209,7 @@ class TiefblueClient:
 
     def __getstate__(self):
         retained_keys = ["bohrium_url", "tiefblue_url", "project_id", "token",
-                         "prefix"]
+                         "prefix", "sharePath", "userSharePath"]
         return {k: self.__dict__[k] for k in retained_keys}
 
     def __setstate__(self, d):
@@ -225,6 +232,8 @@ class TiefblueClient:
         _raise_error(res, "get storage token")
         self.token = res["data"]["token"]
         self.prefix = res["data"]["path"]
+        self.sharePath = res["data"]["sharePath"]
+        self.userSharePath = res["data"]["userSharePath"]
 
     def upload(self, key, path, **kwargs):
         import tiefblue
