@@ -3,7 +3,7 @@ import os
 import re
 import sys
 from copy import deepcopy
-from typing import Any, Dict, List, Union
+from typing import Optional, Any, Dict, List, Union
 
 import jsonpickle
 
@@ -140,7 +140,7 @@ def argo_sequence(
         count: Union[int, ArgoVar] = None,
         start: Union[int, ArgoVar] = None,
         end: Union[int, ArgoVar] = None,
-        format: str = None,
+        format: Optional[str] = None,
 ) -> V1alpha1Sequence:
     """
     Return a numeric sequence in Argo
@@ -265,21 +265,21 @@ class Step:
             parameters: Dict[str, Any] = None,
             artifacts: Dict[str, Union[S3Artifact,
                                        InputArtifact, OutputArtifact]] = None,
-            when: str = None,
+            when: Optional[str] = None,
             with_param: Union[str, list,
                               InputParameter, OutputParameter] = None,
             continue_on_failed: bool = False,
-            continue_on_num_success: int = None,
-            continue_on_success_ratio: float = None,
-            with_sequence: V1alpha1Sequence = None,
-            key: str = None,
-            executor: Executor = None,
-            use_resource: Resource = None,
-            util_image: str = None,
-            util_image_pull_policy: str = None,
+            continue_on_num_success: Optional[int] = None,
+            continue_on_success_ratio: Optional[float] = None,
+            with_sequence: Optional[V1alpha1Sequence] = None,
+            key: Optional[str] = None,
+            executor: Optional[Executor] = None,
+            use_resource: Optional[Resource] = None,
+            util_image: Optional[str] = None,
+            util_image_pull_policy: Optional[str] = None,
             util_command: Union[str, List[str]] = None,
-            parallelism: int = None,
-            slices: Slices = None,
+            parallelism: Optional[int] = None,
+            slices: Optional[Slices] = None,
             register_output_artifacts: Dict[str, dict] = None,
             **kwargs,
     ) -> None:
@@ -951,6 +951,21 @@ class Step:
             if v is None:
                 del self.inputs.artifacts[k]
                 self.template.inputs.artifacts[k].optional = True
+            elif isinstance(v, (list, tuple)):
+                self.template = deepcopy(self.template)
+                for i, a in enumerate(v):
+                    vn = "dflow_%s_%s" % (k, i)
+                    self.template.inputs.artifacts[vn] = deepcopy(
+                        self.template.inputs.artifacts[k])
+                    self.template.inputs.artifacts[vn].path = \
+                        "%s/inputs/artifacts/%s" % (self.template.tmp_root, vn)
+                    self.inputs.artifacts[vn] = deepcopy(
+                        self.template.inputs.artifacts[vn])
+                    self.inputs.artifacts[vn].source = a
+                del self.template.inputs.artifacts[k]
+                del self.inputs.artifacts[k]
+                self.template.n_parts[k] = len(v)
+                self.template.render_script()
             else:
                 self.inputs.artifacts[k].source = v
                 if config["save_path_as_parameter"]:
@@ -977,6 +992,24 @@ class Step:
                             InputParameter(
                             value=v.template.inputs.parameters[
                                 "dflow_%s_path_list" % v.name])
+                if config["lineage"] and k[:6] != "dflow_":
+                    if isinstance(v, S3Artifact):
+                        self.inputs.parameters["dflow_%s_urn" % k] = \
+                            InputParameter(value=v.urn)
+                    elif isinstance(v, OutputArtifact) and v.step is not None \
+                            and "dflow_%s_urn" % v.name in \
+                                v.step.outputs.parameters:
+                        self.inputs.parameters["dflow_%s_urn" % k] = \
+                            InputParameter(
+                            value=v.step.outputs.parameters[
+                                "dflow_%s_urn" % v.name])
+                    elif isinstance(v, InputArtifact) and v.template is not \
+                        None and "dflow_%s_urn" % v.name in \
+                            v.template.inputs.parameters:
+                        self.inputs.parameters["dflow_%s_urn" % k] = \
+                            InputParameter(
+                            value=v.template.inputs.parameters[
+                                "dflow_%s_urn" % v.name])
 
     def prepare_argo_arguments(self, context=None):
         self.argo_parameters = []
