@@ -50,8 +50,8 @@ def get_key(artifact, raise_error=True):
 def download_artifact(
         artifact,
         extract: bool = True,
-        sub_path: str = None,
-        slice: int = None,
+        sub_path: Optional[str] = None,
+        slice: Optional[int] = None,
         **kwargs,
 ) -> List[str]:
     """
@@ -114,6 +114,8 @@ def download_artifact(
 def upload_artifact(
         path: Union[os.PathLike, List[os.PathLike], Set[os.PathLike]],
         archive: str = "default",
+        namespace: Optional[str] = None,
+        name: Optional[str] = None,
         **kwargs,
 ) -> S3Artifact:
     """
@@ -182,7 +184,12 @@ def upload_artifact(
             key = upload_s3(path=tmpdir, **kwargs)
 
     logging.debug("upload artifact: finished")
-    return S3Artifact(key=key, path_list=path_list)
+
+    urn = ""
+    if config["lineage"] and namespace is not None and name is not None:
+        urn = config["lineage"].register_artifact(namespace, name, key)
+
+    return S3Artifact(key=key, path_list=path_list, urn=urn)
 
 
 def copy_artifact(src, dst, sort=False) -> S3Artifact:
@@ -229,7 +236,7 @@ def get_md5(f):
 
 def download_s3(
         key: str,
-        path: os.PathLike = None,
+        path: Optional[os.PathLike] = None,
         recursive: bool = True,
         skip_exists: bool = False,
         **kwargs,
@@ -266,8 +273,8 @@ def download_s3(
 
 def upload_s3(
         path: os.PathLike,
-        key: str = None,
-        prefix: str = None,
+        key: Optional[str] = None,
+        prefix: Optional[str] = None,
         **kwargs,
 ) -> str:
     if s3_config["storage_client"] is not None:
@@ -523,26 +530,36 @@ def run_command(
     return return_code, out, err
 
 
-def find_subclass(module, cls):
-    ops = []
-    for _, m in inspect.getmembers(module):
-        if inspect.isclass(m) and issubclass(m, cls) and m != cls and \
-                m not in ops:
-            ops.append(m)
-    if hasattr(module, "__path__"):
-        for path in module.__path__:
+def subclass_or_none(m, cls):
+    if inspect.isclass(m) and issubclass(m, cls) and m != cls:
+        return m
+    elif isinstance(m, cls) and m.__class__ != cls:
+        return m.__class__
+    else:
+        return None
+
+
+def find_subclass(pkg, cls):
+    subclasses = []
+    for _, m in inspect.getmembers(pkg):
+        s = subclass_or_none(m, cls)
+        if s is not None and s not in subclasses:
+            subclasses.append(m)
+    if hasattr(pkg, "__path__"):
+        for path in pkg.__path__:
             for dir, _, _ in os.walk(path):
-                pkg = (module.__name__ + dir[len(path):]).replace("/", ".")
+                subpkg = (pkg.__name__ + dir[len(path):]).replace("/", ".")
                 for _, name, _ in pkgutil.iter_modules([dir]):
                     try:
-                        mod = __import__(pkg + "." + name, fromlist=pkg)
+                        mod = __import__(subpkg + "." + name, fromlist=subpkg)
                         for _, m in inspect.getmembers(mod):
-                            if inspect.isclass(m) and issubclass(m, cls) and \
-                                    m != cls and m not in ops:
-                                ops.append(m)
-                    except Exception:
-                        pass
-    return ops
+                            s = subclass_or_none(m, cls)
+                            if s is not None and s not in subclasses:
+                                subclasses.append(s)
+                    except Exception as e:
+                        logging.warning("Fail to inspect submodule %s: %s" % (
+                            name, e))
+    return subclasses
 
 
 def linktree(src, dst, func=os.symlink):
@@ -576,11 +593,11 @@ class StorageClient:
 
 class MinioClient(StorageClient):
     def __init__(self,
-                 endpoint: str = None,
-                 access_key: str = None,
-                 secret_key: str = None,
-                 secure: bool = None,
-                 bucket_name: str = None,
+                 endpoint: Optional[str] = None,
+                 access_key: Optional[str] = None,
+                 secret_key: Optional[str] = None,
+                 secure: Optional[bool] = None,
+                 bucket_name: Optional[str] = None,
                  **kwargs,
                  ) -> None:
         self.client = Minio(
